(in-package :fe)

(defun linear-env-fn (points)
  (points->env #'(lambda (x1 y1 x2 y2)
			(make-env1 #'(lambda (x)
					    (+ (* (- y2 y1) x) y1))
					:min x1 :max x2))
		    points))

(defmacro linear-env (&rest points)
  "Defines an env which interpolates the given points by a linear function. Expects a list of x-y-pairs as (0 y1) ... (1 yn)"
  `(linear-env-fn ',points))

(defun sin-env-fn (points)
  (points->env #'(lambda (x1 y1 x2 y2)
			(make-env1
			 #'(lambda (x)
			     (+ (* (sin (* x pi 0.5)) 
				   (- y2 y1))
				y1))
			 :min x1 :max x2))
		    points))

(defmacro sin-env (&rest points)
  "Defines an env which interpolates the given points by a sin function. Unsing only the intervals [0,pi/2] and [pi, 3pi/4] results in edges. Expects a list of x-y-pairs as (0 y1) ... (1 yn)."
  `(sin-env-fn ',points))

(defun sin-env1-fn (points)
  (points->env #'(lambda (x1 y1 x2 y2)
			(make-env1
			 #'(lambda (x)
			     (+ (* (+ (* (sin (- (* x pi) (* pi 0.5)))
					 0.5)
				      0.5)
				   (- y2 y1))
				y1))
			 :min x1 :max x2))
		    points))

(defmacro sin-env1 (&rest points)
  "Defines an env which interpolates the given points by a sin function without clear edges. Expects a list of x-y-pairs as (0 y1) ... (1 yn)."
  `(sin-env1-fn ',points))

;; !!! noch voellig falsch: die Steilheit stimmt nicht !!!
(defun expon-env-fn (points)
  (points->env #'(lambda (x1 x2 y1 y2)
			(make-env1 #'(lambda (x)
					    (+ (expt (/ y2 y1) x) y1 -1))
					:min x1 :max x2))
		    points))

(defmacro expon-env (&rest points)
  "Defines an envelope described be exponential functions. Expects a list of x-y-pairs as (0 y1) ... (1 yn). y values can not be negative."
  `(expon-env-fn ',points))
